use std::future::{ready, Ready};

use actix_identity::IdentityExt;
use actix_web::{
    body::EitherBody,
    dev::{self, Service, ServiceRequest, ServiceResponse, Transform},
    http, Error, HttpResponse,
};
use futures_util::future::LocalBoxFuture;

pub struct RedirectToLogin;

impl<S, B> Transform<S, ServiceRequest> for RedirectToLogin
where
    S: Service<ServiceRequest, Response = ServiceResponse<B>, Error = Error>,
    S::Future: 'static,
    B: 'static,
{
    type Response = ServiceResponse<EitherBody<B>>;
    type Error = Error;
    type InitError = ();
    type Transform = RedirectToLoginMiddleware<S>;
    type Future = Ready<Result<Self::Transform, Self::InitError>>;

    fn new_transform(&self, service: S) -> Self::Future {
        ready(Ok(RedirectToLoginMiddleware { service }))
    }
}
pub struct RedirectToLoginMiddleware<S> {
    service: S,
}

impl<S, B> Service<ServiceRequest> for RedirectToLoginMiddleware<S>
where
    S: Service<ServiceRequest, Response = ServiceResponse<B>, Error = Error>,
    S::Future: 'static,
    B: 'static,
{
    type Response = ServiceResponse<EitherBody<B>>;
    type Error = Error;
    type Future = LocalBoxFuture<'static, Result<Self::Response, Self::Error>>;

    dev::forward_ready!(service);

    fn call(&self, request: ServiceRequest) -> Self::Future {
        // Change this to see the change in outcome in the browser.
        // Usually this boolean would be acquired from a password check or other auth verification.
        let is_logged_in = request.get_identity().is_ok();

        // Don't forward to `/login` if we are already on `/login`.
        if !is_logged_in
            && request.path() != "/login"
            && request.path() != "/imprint"
            && !request.path().starts_with("/reset-password")
            && !request.path().starts_with("/register")
            && !request.path().starts_with("/static")
        {
            let (request, _pl) = request.into_parts();
            let next_location = format!("/login?next={}", request.uri());

            let response = HttpResponse::Found()
                .insert_header((http::header::LOCATION, next_location))
                .finish()
                // constructed responses map to "right" body
                .map_into_right_body();

            return Box::pin(async { Ok(ServiceResponse::new(request, response)) });
        }

        let res = self.service.call(request);

        Box::pin(async move {
            // forwarded responses map to "left" body
            res.await.map(ServiceResponse::map_into_left_body)
        })
    }
}
